@@ -1,13 +1,14 @@
-package EAI::Common 0.10;
+package EAI::Common 0.2;
 
-use strict; use feature 'unicode_strings'; use warnings; no warnings 'uninitialized';
-use Exporter qw(import); use EAI::DateUtil qw(get_curdate is_holiday); use Data::Dumper qw(Dumper); use Getopt::Long qw(:config no_ignore_case); use Log::Log4perl qw(get_logger); use MIME::Lite (); use Scalar::Util qw(looks_like_number); use Module::Refresh ();
+use strict;
+use Exporter; use Log::Log4perl qw(get_logger); use EAI::DateUtil; use Data::Dumper; use Getopt::Long qw(:config no_ignore_case); use Scalar::Util qw(looks_like_number);
 # to make use of colored logs with Log::Log4perl::Appender::ScreenColoredLevels on windows we have to use that (special "use" to make this optional on non-win environments)
 BEGIN {
 	if ($^O =~ /MSWin/) {require Win32::Console::ANSI; Win32::Console::ANSI->import();}
 }
 
-our @EXPORT = qw($EAI_WRAP_CONFIG_PATH $EAI_WRAP_SENS_CONFIG_PATH %common %config %execute @loads @optload %opt readConfigFile getSensInfo setupConfigMerge getOptions setupEAIWrap dumpFlat extractConfigs checkHash checkParam getLogFPathForMail getLogFPath MailFilter setErrSubject setupLogging checkStartingCond sendGeneralMail looks_like_number get_logger);
+our @ISA = qw(Exporter);
+our @EXPORT = qw($EAI_WRAP_CONFIG_PATH $EAI_WRAP_SENS_CONFIG_PATH %common %config %execute @loads @optload %opt readConfigFile getSensInfo setupConfigMerge getOptions setupEAIWrap extractConfigs checkHash getLogFPathForMail getLogFPath MailFilter setErrSubject setupLogging checkStartingCond sendGeneralMail looks_like_number get_logger);
 
 my %hashCheck = (
 	common => {
@@ -18,19 +19,19 @@ my %hashCheck = (
 		task => {},
 	},
 	config => { # parameter category for site global settings, defined in site.config and other associated configs loaded at INIT
-		checkLookup => {"test.pl" => {errmailaddress => "",errmailsubject => "",timeToCheck =>"", freqToCheck => "", logFileToCheck => "", logcheck => "",logRootPath =>""}}, # used for logchecker, each entry of the hash defines a log to be checked, defining errmailaddress to receive error mails, errmailsubject, timeToCheck as earliest time to check for existence in log, freqToCheck as frequency of checks (daily/monthly/etc), logFileToCheck as the name of the logfile to check, logcheck as the regex to check in the logfile and logRootPath as the folder where the logfile is found. lookup key: $execute{scriptname} + $execute{addToScriptName}
+		checkLookup => {"test.pl" => {errmailaddress => "",errmailsubject => "",timeToCheck =>, freqToCheck => "", logFileToCheck => "", logcheck => "",logRootPath =>""},}, # used for logchecker, each entry of the hash defines a log to be checked, defining errmailaddress to receive error mails, errmailsubject, timeToCheck as earliest time to check for existence in log, freqToCheck as frequency of checks (daily/monthly/etc), logFileToCheck as the name of the logfile to check, logcheck as the regex to check in the logfile and logRootPath as the folder where the logfile is found. lookup key: $execute{scriptname} + $execute{addToScriptName}
 		errmailaddress => "", # default mail address for central logcheck/errmail sending 
 		errmailsubject => "", # default mail subject for central logcheck/errmail sending 
 		executeOnInit => "", # code to be executed during INIT of EAI::Wrap to allow for assignment of config/execute parameters from commandline params BEFORE Logging!
-		folderEnvironmentMapping => {Test => "Test", Dev => "Dev", "" => "Prod"}, # Mapping for $execute{envraw} to $execute{env}:
+		folderEnvironmentMapping => {Test => "Test", Dev => "Dev", "" => "Prod"}, # Mapping for $execute{envraw} to $execute{env}
 		fromaddress => "", # from address for central logcheck/errmail sending, also used as default sender address for sendGeneralMail
-		historyFolder => {"" => "default"}, # folders where downloaded files are historized, lookup key as checkLookup, default in "" =>
-		historyFolderUpload => {"" => "default"}, # folders where uploaded files are historized, lookup key as checkLookup, default in "" =>
-		logCheckHoliday => "", # calendar for business days in central logcheck/errmail sending. builtin calendars are AT (Austria), TG (Target), UK (United Kingdom) and WE (for only weekends). Calendars can be added with EAI::DateUtil::addCalendar
+		historyFolder => {"" => "default",}, # folders where downloaded files are historized, lookup key as checkLookup, default in "" =>
+		historyFolderUpload => {"" => "default",}, # folders where uploaded files are historized, lookup key as checkLookup, default in "" =>
+		logCheckHoliday => "", # calendar for business days in central logcheck/errmail sending
 		logs_to_be_ignored_in_nonprod => '', # logs to be ignored in central logcheck/errmail sending
-		logRootPath => {"" => "default"}, # paths to log file root folders (environment is added to that if non production), lookup key as checkLookup, default in "" =>
-		redoDir => {"" => "default"}, # folders where files for redo are contained, lookup key as checkLookup, default in "" =>
-		sensitive => {"prefix" => {user=>"",pwd =>"",hostkey=>"",privkey =>""}}, # hash lookup for sensitive access information in DB and FTP (lookup keys are set with DB{prefix} or FTP{prefix}), may also be placed outside of site.config; all sensitive keys can also be environment lookups, e.g. hostkey=>{Test => "", Prod => ""} to allow for environment specific setting
+		logRootPath => {"" => "default",}, # paths to log file root folders (environment is added to that if non production), lookup key as checkLookup, default in "" =>
+		redoDir => {"" => "default",}, # folders where files for redo are contained, lookup key as checkLookup, default in "" =>
+		sensitive => {"prefix" => {user=>"",pwd =>"",hostkey=>"",privkey =>""},}, # hash lookup for sensitive access information in DB and FTP (lookup keys are set with DB{prefix} or FTP{prefix}), may also be placed outside of site.config; all sensitive keys can also be environment lookups, e.g. hostkey=>{Test => "", Prod => ""} to allow for environment specific setting
 		smtpServer => "", # smtp server for den (error) mail sending
 		smtpTimeout => 60, # timeout for smtp response
 		testerrmailaddress => '', # error mail address in non prod environment
@@ -94,7 +95,7 @@ my %hashCheck = (
 		longreadlen => 1024, # used for setting database handles LongReadLen parameter for DB connection, if not set defaults to 1024
 		noDBTransaction => 1, # don't use a DB transaction for dumpDataIntoDB
 		noDumpIntoDB => 1, # if files from this load should not be dumped to the database
-		postDumpExecs => [{execs => ['',''], condition => ''}], # done in dumpDataIntoDB after postDumpProcessing and before commit/rollback. doInDB everything in execs if condition is fulfilled 
+		postDumpExecs => [{execs => ['',''], condition => ''},], # done in dumpDataIntoDB after postDumpProcessing and before commit/rollback. doInDB everything in execs if condition is fulfilled 
 		postDumpProcessing => "", # done in dumpDataIntoDB after storeInDB, execute perl code in postDumpProcessing
 		postReadProcessing => "", # done in writeFileFromDB after readFromDB, execute perl code in postReadProcessing
 		prefix => "db", # key for sensitive information (e.g. pwd and user) in config{sensitive}
@@ -114,6 +115,7 @@ my %hashCheck = (
 		dontKeepHistory => 1, # if up- or downloaded file should not be moved into historyFolder but be deleted
 		dontMoveIntoHistory => 1, # if up- or downloaded file should not be moved into historyFolder but be kept in homedir
 		emptyOK => 0, # flag to specify whether empty files should not invoke an error message. Also needed to mark an empty file as processed in EAI::Wrap::markProcessed
+		encoding => "", # text encoding of the file in question (e.g. :encoding(utf8))
 		extract => 1, # flag to specify whether to extract files from archive package (zip)
 		extension => "", # the extension of the file to be read (optional, used for redoFile)
 		fieldCode => {}, # additional field based processing code: fieldCode => {field1 => 'perl code', ..}, invoked if key equals either header (as in format_header) or targetheader (as in format_targetheader) or invoked for all fields if key is empty {"" => 'perl code'}. set $skipLineAssignment to true (1) if current line should be skipped from data.
@@ -123,8 +125,6 @@ my %hashCheck = (
 		format_beforeHeader => "", # additional String to be written before the header in write text
 		format_dateColumns => [], # numeric array of columns that contain date values (special parsing) in excel files
 		format_decimalsep => "", # decimal separator used in numbers of sourcefile (defaults to . if not given)
-		format_defaultsep => "", # default separator when format_sep not given (usually in site.config), if not given, "\t" is used as default.
-		format_encoding => "", # text encoding of the file in question (e.g. :encoding(utf8))
 		format_headerColumns => [], # optional numeric array of columns that contain data in excel files (defaults to all columns starting with first column up to format_targetheader length)
 		format_header => "", # format_sep separated string containing header fields (optional in excel files, only used to check against existing header row)
 		format_headerskip => 1, # skip until row-number for checking header row against format_header in excel files
@@ -180,7 +180,7 @@ my %hashCheck = (
 		type => "", # (A)scii or (B)inary
 		user => "", # set user directly, either directly (insecure -> visible) or via sensitive lookup
 	},
-	process => { # used to pass information within each process (data, additionalLookupData, filenames, hadErrors or commandline parameters starting with interactive) and for additional configurations not suitable for DB, File or FTP (e.g. uploadCMD* and onlyExecFor)
+	process => { # used to pass information within each process (data, additionalLookupData, filenames, hadErrors or commandline parameters starting with interactive) and for additional configurations not suitable for DB, File or FTP (e.g. uploadCMD)
 		additionalLookupData => {}, # additional data retrieved from database with EAI::Wrap::getAdditionalDBData
 		archivefilenames => [], # in case a zip archive package is retrieved, the filenames of these packages are kept here, necessary for cleanup at the end of the process
 		data => [], # loaded data: array (rows) of hash refs (columns)
@@ -188,18 +188,15 @@ my %hashCheck = (
 		filesProcessed => {}, # hash for checking the processed files, necessary for cleanup at the end of the whole task
 		hadErrors => 1, # set to 1 if there were any errors in the process
 		interactive_ => "", # interactive options (are not checked), can be used to pass arbitrary data via command line into the script (eg a selected date for the run with interactive_date).
-		onlyExecFor => qr//, # mark loads to only be executed when $common{task}{execOnly} !~ $load->{process}{onlyExecFor}
 		uploadCMD => "", # upload command for use with uploadFileCMD
 		uploadCMDPath => "", # path of upload command
 		uploadCMDLogfile => "", # logfile where command given in uploadCMD writes output (for error handling)
 	},
 	task => { # contains parameters used on the task script level
 		customHistoryTimestamp => "", # optional custom timestamp to be added to filenames moved to History/HistoryUpload/FTP archive, if not given, get_curdatetime is used (YYYYMMDD_hhmmss)
-		execOnly => "", # used to remove loads where $common{task}{execOnly} !~ $load->{process}{onlyExecFor}
 		ignoreNoTest => 0, # ignore the notest file in the process-script folder, usually preventing all runs that are not in production
 		plannedUntil => "2359", # latest time that planned repitition should last
 		redoFile => 1, # flag for specifying a redo
-		redoTimestampPatternPart => "", # part of the regex for checking against filename in redo with additional timestamp/redoDir pattern (e.g. "redo", numbers and _), anything after files barename (and before ".$ext" if extension is defined) is regarded as a timestamp. Example: '[\d_]', the regex is built like ($ext ? qr/$barename($redoTimestampPatternPart|$redoDir)*\.$ext/ : qr/$barename($redoTimestampPatternPart|$redoDir)*.*/)
 		retrySecondsErr => 60, # retry period in case of error
 		retrySecondsErrAfterXfails => 600, # after fail count is reached this alternate retry period in case of error is applied. If 0/undefined then job finishes after fail count
 		retrySecondsXfails => 3, # fail count after which the retrySecondsErr are changed to retrySecondsErrAfterXfails
@@ -216,7 +213,6 @@ my %ignoreType = (
 	Fileformat_skip => 1, # can be line number (int) or "skip until pattern" string
 	Fileformat_sep => 1, # can be regex split or separator string
 	taskskipHolidays => 1, # can be true (1) or calendar string
-	taskskipForFirstBusinessDate => 1, # can be true (1) or calendar string
 );
 
 our %common;our %config;our @loads;our %execute;our @optload;our %opt;
@@ -297,16 +293,6 @@ sub setupConfigMerge {
 		# merge common and cmdline option overrides into loads
 		$loads[$i]={DB=>{%{$common{DB}},%{$loads[$i]{DB}},%{$optload[$i]{DB}}},FTP=>{%{$common{FTP}},%{$loads[$i]{FTP}},%{$optload[$i]{FTP}}},File=>{%{$common{File}},%{$loads[$i]{File}},%{$optload[$i]{File}}},process=>{%{$common{process}},%{$loads[$i]{process}},%{$optload[$i]{process}}}};
 	}
-	# remove load elements where $common{task}{execOnly} !~ $load->{process}{onlyExecFor}
-	my $i=0;
-	while ($i <= $#loads) {
-		if ($common{task}{execOnly} and $common{task}{execOnly} !~ $loads[$i]{process}{onlyExecFor}) {
-			$logger->debug("removing load $i because \$common{task}{execOnly} given and $common{task}{execOnly} !~ $loads[$i]{process}{onlyExecFor} (\$load{process}{onlyExecFor}), \$#loads: $#loads");
-			splice @loads, $i, 1;
-		} else {
-			$i++;
-		}
-	};
 }
 
 # get options for overriding configured settings
@@ -352,13 +338,10 @@ sub getOptions {
 	}
 }
 
-# extract config hashes (DB,FTP,File,process; task is always part of common, which is always there) from $arg hash and return as list of hashes. The config hashes to be extracted are given in string list @required and returned in @ret
-# side effect: sets error subject to first argument $contextSub if ne ""
-sub extractConfigs ($$$;@) {
-	my ($contextSub,$arg,@required) = @_;
+# extract config hashes (DB,FTP,File,process) from $arg hash and return as list of hashes. The config hashes to be extracted are given in string list @required and returned in @ret
+sub extractConfigs ($$;@) {
+	my ($arg,@required) = @_;
 	my $logger = get_logger();
-	$logger->debug(($contextSub ? "setting err subject $contextSub for " : "").(caller(1))[3]) if caller(1);
-	setErrSubject($contextSub) if $contextSub;
 	my @ret;
 	if (ref($arg) eq "HASH") {
 		for my $req (@required) {
@@ -367,8 +350,7 @@ sub extractConfigs ($$$;@) {
 		}
 		checkHash(\%execute,"execute") or $logger->error($@); # also check always the execute hash ...
 	} else {
-		my $errStr = "no ref to hash passed when calling ".(caller(0))[3].", line ".(caller(0))[2]." in ".(caller(0))[1];
-		$errStr = "no ref to hash passed when calling ".(caller(1))[3].", line ".(caller(1))[2]." in ".(caller(1))[1] if caller(1);
+		my $errStr = "no ref to hash passed when calling ".(caller(1))[3].", line ".(caller(1))[2]." in ".(caller(1))[1];
 		$logger->error($errStr);
 	}
 	return @ret;
@@ -377,10 +359,7 @@ sub extractConfigs ($$$;@) {
 # check config hash passed in $hash for validity against hashCheck (valid key entries are there + their valid value types (examples)). returns 0 on error and exception $@ contains details
 sub checkHash ($$) {
 	my ($hash, $hashName) = @_;
-	# try to backtrack as far as possible, maximum 3 calls (script -> function -> extractConfigs):
-	my $locStr = " when calling ".(caller(0))[3].", line ".(caller(0))[2]." in ".(caller(0))[1];
-	$locStr = " when calling ".(caller(1))[3].", line ".(caller(1))[2]." in ".(caller(1))[1] if caller(1);
-	$locStr = " when calling ".(caller(2))[3].", line ".(caller(2))[2]." in ".(caller(2))[1] if caller(2);
+	my $locStr =  " when calling ".(caller(2))[3].", line ".(caller(2))[2]." in ".(caller(2))[1];
 	eval {
 		for my $defkey (keys %{$hash}) {
 			unless ($hashName eq "process" and $defkey =~ /interactive.*/) {
@@ -389,7 +368,7 @@ sub checkHash ($$) {
 				} else {
 					# check type for existing keys, if explicitly not defined then ignore...
 					if (defined($hash->{$defkey})) {
-						die "wrong reference type for value: \$".$hashName."{".$defkey."}: ".ref($hashCheck{$hashName}{$defkey})." is not the same as the passed one:".ref($hash->{$defkey}).",".$locStr if (ref($hashCheck{$hashName}{$defkey}) ne ref($hash->{$defkey}) && !$ignoreType{$hashName.$defkey});
+						die "wrong reference type for value: \$".$hashName."{".$defkey."},".$locStr if (ref($hashCheck{$hashName}{$defkey}) ne ref($hash->{$defkey}) && !$ignoreType{$hashName.$defkey});
 						die "wrong type for value: \$".$hashName."{".$defkey."},".$locStr if (looks_like_number($hashCheck{$hashName}{$defkey}) ne looks_like_number($hash->{$defkey}) && !$ignoreType{$hashName.$defkey});
 					}
 				}
@@ -400,26 +379,6 @@ sub checkHash ($$) {
 	return 1;
 }
 
-# check parameter passed in $subhash
-sub checkParam ($$) {
-	my ($subhash,$keytoCheck) = @_;
-	my $logger = get_logger();
-	if (ref($subhash) ne "HASH") {
-		$logger->error("passed argument subhash to checkParam is not a hash");
-		return 0;
-	}
-	my ($subhashname) = split /=/, Dumper($subhash);
-	$subhashname =~ s/\$//;
-	if (!defined($subhash->{$keytoCheck})) {
-		$logger->error("key $keytoCheck not defined in subhash ".dumpFlat($subhash,1));
-		return 0;
-	} elsif (!$subhash->{$keytoCheck} and !looks_like_number($hashCheck{$subhashname}{$keytoCheck})) {
-		$logger->error("value of key $keytoCheck empty in subhash ".dumpFlat($subhash,1));
-		return 0;
-	}
-	return 1;
-}
-
 # path of logfile and path of yesterdays logfile (after rolling) - getLogFPathForMail and getLogFPath can be used in site-wide log.config
 our ($LogFPath, $LogFPathDayBefore);
 sub getLogFPathForMail {
@@ -447,18 +406,14 @@ sub setupLogging {
 	# get logRootPath, historyFolder, historyFolderUpload and redoDir from lookups in config.
 	# if they are not in the script home directory (having an absolute path) then build environment-path separately for end folder (script home directory is already in its environment)
 	for my $foldKey ("redoDir","logRootPath","historyFolder","historyFolderUpload") {
-		warn ("\$config{$foldKey} is not a hash, cannot get $foldKey from it !") if (ref($config{$foldKey}) ne "HASH");
-		my $folder = $config{$foldKey}{$execute{scriptname}.$execute{addToScriptName}} if (ref($config{$foldKey}) eq "HASH");
+		my $folder = $config{$foldKey}{$execute{scriptname}.$execute{addToScriptName}};
 		my $defaultFolder;
 		if (!$folder) {
 			$folder = $config{$foldKey}{""}; # take default, if no lookup defined for script
 			$defaultFolder = 1;
 		}
-		$folder = $opt{config}{$foldKey} if $opt{config}{$foldKey}; # also possible to override these settings via options
-		# convert backslash to slash to avoid problems in string handling (unicode ...)
-		$folder =~ s/\\/\//g;
-		if ($folder =~ /^(\S:)*\/(.*?)$/ and !$opt{config}{$foldKey}) {
-			my ($folderPath,$endFolder) = ($folder =~ /(.*)\/(.*?)$/); # slash acts as path separator for last part
+		if ($folder =~ /^(\S:)*[\\|\/](.*?)$/) {
+			my ($folderPath,$endFolder) = ($folder =~ /(.*)[\\|\/](.*?)$/); # both slash as well as backslash act as path separator for last part
 			# default Folder is built differently: folderPath/endFolder/environ instead of folderPath/environ/endFolder
 			$execute{$foldKey} = ($defaultFolder ? $folderPath."/".$endFolder.($execute{envraw} ? "/".$execute{envraw} : "") : $folderPath.($execute{envraw} ? "/".$execute{envraw} : "")."/".$endFolder);
 		} else {
@@ -497,9 +452,6 @@ sub setupLogging {
 			$logger->error("no errmailaddress found for ".$execute{scriptname}.$execute{addToScriptName}.", no entry found in \$config{checkLookup}{$execute{scriptname}.$execute{addToScriptName}}");
 		}
 		setErrSubject("Setting up EAI.Wrap"); # general context after logging initialization: setup of EAI.Wrap by script
-	} else {
-		# remove any defined mail appenders
-		undef(Log::Log4perl->appenders()->{"MAIL"}) if Log::Log4perl->appenders()->{"MAIL"};
 	}
 }
 
@@ -508,35 +460,29 @@ sub setupEAIWrap {
 	my $logger = get_logger();
 	setupConfigMerge(); # %config (from site.config, amended with command line options) and %common (from process script, amended with command line options) are merged into %common and all @loads (amended with command line options)
 	# starting log entry: process script name + %common parameters, used for process monitoring (%config is not written due to sensitive information)
+	$Data::Dumper::Indent = 0; # temporarily flatten dumper output for single line
+	$Data::Dumper::Sortkeys = 1; # sort keys to get outputs easier to read
+	my $configdump = Dumper(\%common);
+	$configdump =~ s/\s+//g;$configdump =~ s/\$VAR1=//;$configdump =~ s/,'/,/g;$configdump =~ s/{'/{/g;$configdump =~ s/'=>/=>/g; # compress information
+	my $exedump = Dumper(\%execute);
+	$exedump =~ s/\s+//g;$exedump =~ s/\$VAR1=//;$exedump =~ s/,'/,/g;$exedump =~ s/{'/{/g;$exedump =~ s/'=>/=>/g; # compress information
 	$logger->info("==============================================================================================");
-	$logger->info("started $execute{scriptname} in $execute{homedir} (environment $execute{env}) ... execute parameters: ".dumpFlat(\%execute,1,1)." ... common parameters: ".dumpFlat(\%common,1,1));
+	$logger->info("started $execute{scriptname} in $execute{homedir} (environment $execute{env}), execute parameters: $exedump common parameters: $configdump");
 	if ($logger->is_debug) {
-		$logger->debug("load $_ parameters: ".dumpFlat($loads[$_],1,1)) for (0..$#loads);
-		$logger->trace("config parameters: ".dumpFlat(\%config,1,1)) if !defined($config{sensitive}) and $logger->is_trace();
+		for my $i (0..$#loads) {
+			my $loaddump = Dumper($loads[$i]);
+			$loaddump =~ s/\s+//g;$loaddump =~ s/\$VAR1=//;$loaddump =~ s/,'/,/g;$loaddump =~ s/{'/{/g;$loaddump =~ s/'=>/=>/g; # compress information
+			$logger->debug("load $i parameters: $loaddump");
+		}
 	}
+	$Data::Dumper::Indent = 2;
 	# check starting conditions and exit if met (returned true)
 	checkStartingCond(\%common) and exit 0;
 	setErrSubject("General EAI.Wrap script execution"); # general context after setup of EAI.Wrap
 }
 
-# returned Data::Dumpered datastructure given in $arg flattened, sorted (if $sortDump given) and compressed (if $compressDump given)
-sub dumpFlat ($;$$) {
-	my $arg = shift;
-	my $sortDump = shift;
-	my $compressDump = shift;
-	$Data::Dumper::Indent = 0; # temporarily flatten dumper output for single line
-	$Data::Dumper::Sortkeys = 1 if $sortDump; # sort keys to get outputs easier to read
-	my $dump = Dumper($arg);
-	if ($compressDump) {
-		$dump =~ s/\s+//g;$dump =~ s/,'/,/g;$dump =~ s/{'/{/g;$dump =~ s/'=>/=>/g; # compress information
-	}
-	$dump =~ s/\$VAR1=//;
-	$Data::Dumper::Indent = 2;
-	return $dump;
-}
-
 # refresh modules and logging config for changes
-sub refresh {
+sub refresh() {
 	# refresh modules to enable correction of processing without restart
 	Module::Refresh->refresh;
 	# also check for changes in logging configuration
@@ -547,15 +493,15 @@ sub refresh {
 sub checkStartingCond ($) {
 	my $arg = shift;
 	my $logger = get_logger();
-	my ($task) = extractConfigs("checking starting conditions",$arg,"task");
+	my ($task) = extractConfigs($arg,"task");
+
 	my $curdate = get_curdate();
-	$logger->debug("checkStartingCond for \$curdate: $curdate, task config:".dumpFlat($task,1));
 	# skipHolidays is either a calendar or 1 (then defaults to $task->{skipHolidaysDefault})
 	my $holidayCal = $task->{skipHolidays} if $task->{skipHolidays};
 	# skipForFirstBusinessDate is for "wait with execution for first business date", either this is a calendar or 1 (then calendar is skipHolidaysDefault), this cannot be used together with skipHolidays
 	$holidayCal = $task->{skipForFirstBusinessDate} if $task->{skipForFirstBusinessDate};
 	# default setting (1 becomes $task->{skipHolidaysDefault})
-	$holidayCal = $task->{skipHolidaysDefault} if ($task->{skipForFirstBusinessDate} eq "1" or $task->{skipHolidays} eq "1");
+	$holidayCal = $task->{skipHolidaysDefault} if ($task->{skipForFirstBusinessDate} == 1 or $task->{skipHolidays} == 1);
 	if ($holidayCal) {
 		if (is_holiday($holidayCal,$curdate) and !$task->{redoFile}) {
 			$logger->info("skip processing (skipHolidays = ".$task->{skipHolidays}.", skipForFirstBusinessDate = ".$task->{skipForFirstBusinessDate}.") as $curdate holiday in $holidayCal !");
@@ -574,7 +520,6 @@ sub checkStartingCond ($) {
 		for (1..$daysfrom1st) {
 			$nonBusinessDays += (is_weekend(subtractDays($curdate,$_)) or is_holiday($holidayCal,subtractDays($curdate,$_)));
 		}
-		$logger->debug("\$nonBusinessDays: $nonBusinessDays,\$daysfrom1st: $daysfrom1st");
 		if ($nonBusinessDays < $daysfrom1st) {
 			$logger->info("skip processing (skipForFirstBusinessDate = ".$task->{skipForFirstBusinessDate}.") as processing already took place on a business day before $curdate!");
 			return 1;
@@ -592,7 +537,6 @@ sub checkStartingCond ($) {
 sub sendGeneralMail ($$$$$$;$$$$) {
 	my ($From, $To, $Cc, $Bcc, $Subject, $Data, $Type, $Encoding, $AttachType, $AttachFile) = @_;
 	my $logger = get_logger();
-	$logger->error("cannot send mail as \$config{smtpServer} not set") if !$config{smtpServer};
 	$logger->info("sending general mail From:".($From ? $From : $config{fromaddress}).", To:".($execute{envraw} ? $config{testerrmailaddress} : $To).", CC:".($execute{envraw} ? "" : $Cc).", Bcc:".($execute{envraw} ? "" : $Bcc).", Subject:".($execute{envraw} ? $execute{envraw}.": " : "").$Subject.", Type:".($Type ? $Type : "TEXT").", Encoding:".($Type eq 'multipart/related' ? undef : $Encoding).", AttachType:$AttachType, AttachFile:$AttachFile ...");
 	$logger->debug("Mailbody: $Data");
 	my $msg = MIME::Lite->new(
@@ -605,7 +549,7 @@ sub sendGeneralMail ($$$$$$;$$$$) {
 			Data    => ($Type eq 'multipart/related' ? undef : $Data),
 			Encoding => ($Type eq 'multipart/related' ? undef : $Encoding)
 		);
-	$logger->error("couldn't create msg for mail sending..") unless $msg;
+	$logger->error("couldn't create msg  for mail sending..") unless $msg;
 	if ($Type eq 'multipart/related') {
 		$msg->attach(
 			Type => 'text/html',
@@ -627,33 +571,12 @@ sub sendGeneralMail ($$$$$$;$$$$) {
 			Path => $AttachFile
 		);
 	}
-	$msg->send();
+	$msg->send('smtp', $config{smtpServer}, AuthUser=>$config{smtpAuth}{user}, AuthPass=>$config{smtpAuth}{pwd});
 	if ($msg->last_send_successful()) {
 		$logger->info("Mail sent");
 		$logger->trace("sent message: ".$msg->as_string) if $logger->is_trace();
 	}
 }
-
-package Log::Dispatch::Email::LogSender;
- 
-use Log::Dispatch::Email;
-use base qw( Log::Dispatch::Email );
- 
-sub send_email {
-	my $self = shift;
-	my %p    = @_;
-	# catch wide non utf8 characters to avoid die in MIME::Lite
-	eval {decode('UTF-8',$p{message},$Encode::FB_CROAK )} or $p{message} =~ s/[^\x00-\x7f]/?/g;
-	my $msg = MIME::Lite->new(
-			From    => $self->{from},
-			To      => ( join ',', @{ $self->{to} } ),
-			Subject => $self->{subject},
-			Type    => "TEXT",
-			Data    => $p{message},
-		);
-	eval {$msg->send();} or warn("couldn't send error mail: $@");
-}
-
 1;
 __END__
 
@@ -666,24 +589,22 @@ EAI::Common - Common parts for the EAI::Wrap package
  %config .. hash for global config (set in $EAI_WRAP_CONFIG_PATH/site.config, amended with $EAI_WRAP_CONFIG_PATH/additional/*.config)
  %common .. common load configs for the task script
  @loads .. list of hashes defining specific load processes
- %execute .. hash of parameters for current running task script
+ %execute .. hash of parameters for current running task script (having one or multiple loads)
 
- readConfigFile ($configfilename)
- getSensInfo ($prefix, $key)
- setupConfigMerge ()
- getOptions ()
- extractConfigs ($contextSub, $arg, @required)
- checkHash ($hash, $hashName)
- checkParam ($subhash, $hashName)
- setupEAIWrap ()
- dumpFlat ($arg, $sortDump, $compressDump)
- getLogFPathForMail ()
- getLogFPath ()
- MailFilter ()
- setErrSubject ($context)
- setupLogging ()
- checkStartingCond ($task)
- sendGeneralMail ($From, $To, $Cc, $Bcc, $Subject, $Type, $Data, $Encoding, $AttachType, $AttachFile)
+ readConfigFile
+ getSensInfo
+ setupConfigMerge
+ getOptions
+ extractConfigs
+ checkHash
+ setupEAIWrap
+ getLogFPathForMail
+ getLogFPath
+ MailFilter
+ setupLogging
+ setErrSubject $context
+ checkStartingCond $process
+ sendGeneralMail $From, $To, $Cc, $Bcc, $Subject, $Type, $Data, $Encoding, $AttachType, $AttachFile
 
 =head1 DESCRIPTION
 
@@ -699,9 +620,7 @@ read given config file (eval perl code in site.config and related files)
 
 =item getSensInfo ($$)
 
-arguments are $prefix and $key
-
-get sensitive info from $config{sensitive}{$prefix}{$key}, depending on queried key being a ref to hash, get the environment lookup hash key value or the value directly
+get sensitive info from $config{sensitive}{$prefix}{$key}, arguments are $prefix and $key, depending on queried key being a ref to hash, get the environment lookup hash key value or the value directly
 
 =item setupConfigMerge
 
@@ -709,7 +628,7 @@ setupConfigMerge creates cascading inheritance of config/DB/File/FTP/process/tas
 
  %config <-- config options from command line
  - is merged into -->
- %common (common task parameters defined in script) <-- DB, FTP, File, task and process options from command line
+ %common (common task parameters defined in script)
  - is merged into -->
  $loads[] <-- DB, FTP, File and process options from command line
 
@@ -717,23 +636,13 @@ setupConfigMerge creates cascading inheritance of config/DB/File/FTP/process/tas
 
 get options for overriding configured settings, results are stored in globally available hash %opt and list @optloads
 
-=item extractConfigs ($$$;@)
+=item extractConfigs ($$;@)
 
-arguments are $contextSub, $arg and @required.
-
-sets error subject to $contextSub (first argument) and extracts config hashes (DB,FTP,File,process,task) from ref to hash $arg (second argument) and return them as a list of hashes. The config hashes to be extracted are given as strings in the parameter list @required (at least one is required).
+extract config hashes (DB,FTP,File,process,task) from $arg ref to hash (first argument) and return as list of hashes. The config hashes to be extracted are given in the following parameter list @required.
 
 =item checkHash ($$)
 
-arguments are $hash and $hashName
-
-check keys of config subhash (named $hashName) being passed in $hash for validity against internal hash %hashCheck (valid key entries + their valid value types (examples) are defined there). returns 0 on error and exception $@ contains details, to allow for checkHash(..) or {handle exception}.
-
-=item checkParam ($$)
-
-arguments are $subhash and $hashName
-
-check existence of parameter $hashName within first argument $subhash, returns 0 if not defined or empty (only non-numerics)
+check config hash passed in $hash for validity against hashCheck (valid key entries are there + their valid value types (examples)). returns 0 on error and exception $@ contains details, to allow for checkHash(..) or {handle exception}
 
 =item setupEAIWrap
 
@@ -743,12 +652,6 @@ Correctness of the configuration and starting conditions are also checked, preve
 
 following three functions can be used in the central log.config as coderefs for callback.
 
-=item dumpFlat ($;$$)
-
-arguments are $arg: datastructure to be flat dumped, $sortDump: sort all keys in $arg and $compressDump: compress information in $arg
-
-returns Data::Dumper dumped datastructure given in $arg flattened, sorted (if $sortDump given) and compressed (if $compressDump given)
-
 =item getLogFPathForMail
 
 for custom conversion specifiers: returns path of configured logfile resp logfile of previous day (as a file:// hyperlink)
@@ -763,10 +666,10 @@ for Mail appender config: used for filtering if further mails should be sent, co
 
 =item setErrSubject ($)
 
-argument $context .. text for context of subject
-
 set context specific subject for ErrorMail
 
+ $context .. text for context of subject
+
 =item setupLogging
 
 set up logging from site.config information (potentially split up using additional configs) and the central log.config. Important configs for logging in the config hash are logRootPath (direct or environment lookup setting for the log root folder), errmailaddress (default address for sending mails in case of error), errmailsubject (subject for error mails, can be changed with L<setErrSubject|/setErrSubject>), testerrmailaddress (default address for sending mails in case of error in non production environments), smtpServer (for error and other mail sending), smtpTimeout and checkLookup.
@@ -777,26 +680,12 @@ Each entry of the sub-hash defines defines the errmailaddress to receive error m
 
 =item checkStartingCond ($)
 
-argument $task .. config information
-
 check starting conditions from process config information and return 1 if met
 
+ $task .. config information
+
 =item sendGeneralMail ($$$$$$;$$$$)
 
-arguments are
-
- $From .. sender
- $To .. recipient
- $Cc .. cc recipient (optional, but need arg)
- $Bcc .. mcc recipient  (optional, but need arg)
- $Subject .. mail subject
- $Data .. the mail body, either plain text or html.
- $Type .. mail mime type (eg text/plain, text/html oder 'multipart/related'), if 'multipart/related', then a ref to array to the filenames (path), that should be attached is expected to be set in $AttachFile. 
- In the above example a mail body ($Data) is being set as the first attachment and its type is text/html. The rest of the attachments from $AttachFile are encoded using base64 and all have mime type AttachType (see below).
- $Encoding .. encoding for mail body, optional (eg quoted-printable)
- $AttachType .. mime type for attachment(s) (eg text/csv or image/png), optional
- $AttachFile .. file name/path(s) for attachment(s), optional (hat to be ref to array, if $Type = 'multipart/related')
-
 send general mail, either simple text or html mails, mails with an attachment or multipart mails for "in-body" attachments (eg pictures). 
 In this case the mail body needs to be HTML, attachments are referred to inside the HTML code and are passed as a ref to array of paths in $AttachFile.
  
@@ -810,6 +699,20 @@ Example:
  my @filelist = glob("*.png");
  sendGeneralMail(undef,'address@somewhere.com',undef,undef,"subject for mail",$body,'multipart/related','quoted-printable','image/png',\@filelist);
 
+parameters for sendGeneralMail
+
+ $From .. sender
+ $To .. recipient
+ $Cc .. cc recipient (optional, but need arg)
+ $Bcc .. mcc recipient  (optional, but need arg)
+ $Subject .. mail subject
+ $Data .. the mail body, either plain text or html.
+ $Type .. mail mime type (eg text/plain, text/html oder 'multipart/related'), if 'multipart/related', then a ref to array to the filenames (path), that should be attached is expected to be set in $AttachFile. 
+ In the above example a mail body ($Data) is being set as the first attachment and its type is text/html. The rest of the attachments from $AttachFile are encoded using base64 and all have mime type AttachType (see below).
+ $Encoding .. encoding for mail body, optional (eg quoted-printable)
+ $AttachType .. mime type for attachment(s) (eg text/csv or image/png), optional
+ $AttachFile .. file name/path(s) for attachment(s), optional (hat to be ref to array, if $Type = 'multipart/related')
+
 =back
 
 =head1 COPYRIGHT
