@@ -1,22 +1,32 @@
-package EAI::DB 0.10;
+package EAI::DB 0.2;
 
-use strict; use feature 'unicode_strings'; use warnings;
-use Exporter qw(import); use DBI qw(:sql_types); use DBD::ODBC (); use Data::Dumper qw(Dumper); use Log::Log4perl qw(get_logger);
+use strict;
+use DBI qw(:sql_types); use DBD::ODBC; use Data::Dumper; use Log::Log4perl qw(get_logger); use Exporter;
 
-our @EXPORT = qw(newDBH beginWork commit rollback readFromDB readFromDBHash doInDB storeInDB deleteFromDB updateInDB getConn setConn);
+our @ISA = qw(Exporter);
+our @EXPORT = qw(newDBH beginWork commit rollback readFromDB readFromDBHash doInDB storeInDB deleteFromDB updateInDB);
 
 my $dbh; # module static DBI handle, will be dynamic when using OO-Style here
-my $DSN = ""; # module static DSN string, will be dynamic when using OO-Style here
+my $DSN; # module static DSN string, will be dynamic when using OO-Style here
+
+# used to get the raw db handler, mainly for testability
+sub getConn {
+	return ($dbh, $DSN);
+}
 
 # create a new handle for a database connection
 sub newDBH ($$) {
-	my ($DB,$newDSN) = @_;
+	my ($DB,$execute) = @_;
 	my $logger = get_logger();
+	my ($DSNeval, $newDSN);
+	$DSNeval = $DB->{DSN};
+	$newDSN = eval qq{"$DSNeval"};
+	$logger->error("error parsing \$DB->{DSN}(".$DB->{DSN}.") (couldn't interpolate all values):".$DSNeval) if !$newDSN;
 	if ($DSN ne $newDSN or !defined($dbh)) {
 		$DSN = $newDSN;
 		$logger->debug("DSN: $DSN");
 		$dbh->disconnect() if defined($dbh);
-		$dbh = DBI->connect("dbi:ODBC:$DSN",undef,undef,{PrintError=>0,RaiseError=>0}) or do {
+		$dbh = DBI->connect("dbi:ODBC:$DSN",undef,undef,{PrintError => 0,RaiseError => 0}) or do {
 			$logger->error("DB connection error:".$DBI::errstr.",DSN:".$DSN);
 			undef $dbh;
 			return 0;
@@ -64,11 +74,11 @@ sub rollback {
 
 # read data into array returned in $data
 sub readFromDB ($$) {
-	my ($DB, $data) = @_;
+	my ($param, $data) = @_;
 	my $logger = get_logger();
-	my $statement = $DB->{query};
+	my $statement = $param->{query};
 	eval {
-		die "no ref to hash argument param given ({query=>''})" if ref($DB) ne "HASH";
+		die "no ref to hash argument param given ({query=>''})" if ref($param) ne "HASH";
 		die "no ref to array argument data (for returning data) given" if ref($data) ne "ARRAY";
 		die "no valid dbh connection available" if !defined($dbh);
 		die "no statement (hashkey query) given" if (!$statement);
@@ -77,7 +87,7 @@ sub readFromDB ($$) {
 		eval {
 			my $sth = $dbh->prepare($statement);
 			$sth->execute;
-			@{$DB->{columnnames}} = @{$sth->{NAME}} if $sth->{NAME}; # take field names from the statement handle of query, used for later processing
+			@{$param->{columnnames}} = @{$sth->{NAME}} if $sth->{NAME}; # take field names from the statement handle of query, used for later processing
 			@$data = @{$sth->fetchall_arrayref({})};
 		};
 		die $@.",DB error: ".$DBI::errstr." executed statement: ".$statement if ($@);
@@ -87,19 +97,19 @@ sub readFromDB ($$) {
 		$logger->error($@);
 		return 0;
 	} else {
-		$logger->trace("columns:".Dumper($DB->{columnnames}).",retrieved data:".Dumper($data)) if $logger->is_trace;
+		$logger->trace("columns:".Dumper($param->{columnnames}).",retrieved data:".Dumper($data)) if $logger->is_trace;
 		return 1;
 	}
 }
 
-# read data into hash using columns $DB->{keyfields} as the unique key for the hash (used for lookups), returned in $data
+# read data into hash using columns $param->{keyfields} as the unique key for the hash (used for lookups), returned in $data
 sub readFromDBHash ($$) {
-	my ($DB, $data) = @_;
+	my ($param, $data) = @_;
 	my $logger = get_logger();
-	my $statement = $DB->{query};
-	my @keyfields = $DB->{keyfields} if defined($DB->{keyfields});
+	my $statement = $param->{query};
+	my @keyfields = $param->{keyfields} if defined($param->{keyfields});
 	eval {
-		die "no ref to hash argument param given ({query=>'',keyfields=>[]})" if ref($DB) ne "HASH";
+		die "no ref to hash argument param given ({query=>'',keyfields=>[]})" if ref($param) ne "HASH";
 		die "no ref to hash argument data (for returning data) given" if ref($data) ne "HASH";
 		die "no valid dbh connection available" if !defined($dbh);
 		die "no statement (hashkey query) given" if (!$statement);
@@ -109,7 +119,7 @@ sub readFromDBHash ($$) {
 		eval {
 			my $sth = $dbh->prepare($statement);
 			$sth->execute;
-			@{$DB->{columnnames}} = @{$sth->{NAME}} if $sth->{NAME}; # take field names from the statement handle of query, used for later processing
+			@{$param->{columnnames}} = @{$sth->{NAME}} if $sth->{NAME}; # take field names from the statement handle of query, used for later processing
 			%$data = %{$sth->fetchall_hashref(@keyfields)};
 		};
 		die $@.",DB error: ".$DBI::errstr." executed statement: ".$statement if ($@);
@@ -119,19 +129,19 @@ sub readFromDBHash ($$) {
 		$logger->error($@);
 		return 0;
 	} else {
-		$logger->trace("columns:".Dumper($DB->{columnnames}).",retrieved data:".Dumper($data)) if $logger->is_trace;
+		$logger->trace("columns:".Dumper($param->{columnnames}).",retrieved data:".Dumper($data)) if $logger->is_trace;
 		return 1;
 	}
 }
 
-# do general statement $DB->{doString} in database using optional parameters passed in array ref $DB->{parameters}, optionally passing back values in $data
+# do general statement $param->{doString} in database using optional parameters passed in array ref $param->{parameters}, optionally passing back values in $data
 sub doInDB ($;$) {
-	my ($DB, $data) = @_;
+	my ($param, $data) = @_;
 	my $logger = get_logger();
-	my $doString = $DB->{doString};
-	my @parameters = @{$DB->{parameters}} if $DB->{parameters};
+	my $doString = $param->{doString};
+	my @parameters = @{$param->{parameters}} if $param->{parameters};
 	eval {
-		die "no param hash argument ({doString=>''}) given" if ref($DB) ne "HASH";
+		die "no param hash argument ({doString=>''}) given" if ref($param) ne "HASH";
 		die "no valid dbh connection available" if !defined($dbh);
 		die "no sql statement doString given" if !$doString;
 		$logger->debug("do in DB: $doString, parameters: @parameters");
@@ -172,7 +182,6 @@ sub storeInDB ($$) {
 	my $debugKeyIndicator = $DB->{debugKeyIndicator};
 
 	eval {
-		no warnings 'uninitialized';
 		my @keycolumns = split "AND", $primkey;
 		map { s/=//; s/\?//; s/ //g;} @keycolumns;
 		$logger->debug("tableName:$tableName,addID:$addID,upsert:$upsert,primkey:$primkey,ignoreDuplicateErrs:$ignoreDuplicateErrs,deleteBeforeInsertSelector:$deleteBeforeInsertSelector,incrementalStore:$incrementalStore,doUpdateBeforeInsert:$doUpdateBeforeInsert,debugKeyIndicator:$debugKeyIndicator");
@@ -187,7 +196,7 @@ sub storeInDB ($$) {
 		}
 		die "no schemaName available (neither from tablename containing schema nor from parameter schemaName" if !$schemaName;
 		my $colh = $dbh->column_info('', $schemaName, $tableName, "%");
-		my $coldefs = $colh->fetchall_hashref("COLUMN_NAME");
+		my $coldefs = $colh->fetchall_hashref ("COLUMN_NAME");
 		die "no field definitions found for $schemaName.$tableName using DSN $DSN" if scalar(keys %{$coldefs}) == 0; # no more information can be given as column_info is just a select on information store..
 		$logger->trace("coldefs:\n".Dumper($coldefs)) if $logger->is_trace;
 		my %IDName;
@@ -196,9 +205,9 @@ sub storeInDB ($$) {
 			$IDName{$_} = 1 for (keys %{$addID});
 		}
 		my $i=0; my @columns;
-		$logger->trace("type info:\n".Dumper($dbh->type_info("SQL_ALL_TYPES"))) if $logger->is_trace; # all available data type informations of DBD:ODBC driver
+		$logger->trace("type info:\n".Dumper($dbh->type_info("SQL_ALL_TYPES"))) if $logger->is_trace; # all availably data type informations of DBD:ODBC driver
 		for (keys %{$coldefs}) {
-			if ($coldefs->{$_}{"COLUMN_DEF"} =~ /identity/ or $coldefs->{$_}{"TYPE_NAME"} =~ /identity/) { # for identity (auto incrementing) fields no filling needed
+			if ($coldefs->{$_}{"COLUMN_DEF"} =~ /identity/ || $coldefs->{$_}{"TYPE_NAME"} =~ /identity/) { # for identity (auto incrementing) fields no filling needed
 				$logger->trace("TYPE_NAME for identity field ".$_.':'.$coldefs->{$_}{"TYPE_NAME"}) if $logger->is_trace;
 			} else {
 				$columns[$i]= $_;
@@ -286,7 +295,7 @@ sub storeInDB ($$) {
 							$severity = 1 if !$severity;
 						}
 						if ($dataArray[$tgtCol] && $dataArray[$tgtCol] !~ /^\d{4}\-\d{2}\-\d{2} \d{2}:\d{2}:\d{2}$/ && $dataArray[$tgtCol] !~ /^\d{4}\-\d{2}\-\d{2}$/) {
-							$errorIndicator .= '| correct dateformat (\d{4}\-\d{2}\-\d{2} \d{2}:\d{2}:\d{2} or \d{4}\-\d{2}\-\d{2}) couldn\'t be created for: '.$dataArray[$tgtCol].", field: ".$columns[$dbCol];
+							$errorIndicator .= "| correct dateformat (\d{4}\-\d{2}\-\d{2} \d{2}:\d{2}:\d{2} oder \d{4}\-\d{2}\-\d{2}) couldn't be created for: ".$dataArray[$tgtCol].", field: ".$columns[$dbCol];
 							$dataArray[$tgtCol] = undef;
 							$severity = 1 if !$severity;
 						}
@@ -312,7 +321,7 @@ sub storeInDB ($$) {
 						# fill primary key values with data from current row for update (in case insert fails)
 						$updselector =~ s/$colName\s*=\s*\?/\[$colName\] = $colVal/ if ($updselector =~ /^$colName\s*=\s*\?/ || $updselector =~ /AND $colName\s*=\s*\?/i);
 						# delete before select statement requires specific selector to only delete once for occurred data (first appearance of colVal)
-						$deleteBeforeInsertSelector =~ s/$colName\s*=\s*\?/\[$colName\] = $colVal/ if ($deleteBeforeInsertSelector and ($deleteBeforeInsertSelector =~ /^$colName\s*=\s*\?/ || $deleteBeforeInsertSelector =~ /AND $colName\s*=\s*\?/));
+						$deleteBeforeInsertSelector =~ s/$colName\s*=\s*\?/\[$colName\] = $colVal/ if ($deleteBeforeInsertSelector =~ /^$colName\s*=\s*\?/ || $deleteBeforeInsertSelector =~ /AND $colName\s*=\s*\?/);
 						$updcols.="[".$colName."] = ".$colVal.",";
 						$inscols.="[".$colName."],";
 						$inscolVals.=$colVal.",";
@@ -328,10 +337,9 @@ sub storeInDB ($$) {
 					$logger->info("deleting data from $schemaName.$tableName, criteria: $deleteBeforeInsertSelector");
 					my $dostring = "delete from $schemaName.$tableName WHERE $deleteBeforeInsertSelector";
 					my $affectedRows = $dbh->do($dostring) or die $DBI::errstr." with $dostring ".$debugKeyIndicator;
-					$affectedRows =~ s/E0//;
 					# mark deleteBeforeInsertSelector as executed for these data
 					$beforeInsert{$deleteBeforeInsertSelector} = 1;
-					$logger->info("entering data into $schemaName.$tableName after delete before insert, deleted rows: $affectedRows");
+					$logger->info("entering data into $schemaName.$tableName after delete before insert, deleted rows: $affectedRows ($DBI::errstr)");
 				}
 				if ($logger->is_trace) {
 					$logger->trace("data to be inserted:");
@@ -511,39 +519,18 @@ sub updateInDB ($$) {
 		return 1;
 	}
 }
-
-# set handle with externally created DBI::db connection (if EAI::DB::newDBH capabilities are not sufficient)
-sub setConn ($;$) {
-	my ($handle,$setDSN) = shift;
-	my $logger = get_logger();
-	eval {
-		die "no DBI::db handle passed to setHandle, argument is '".(defined($handle) ? ref($handle) : "undefined")."'" unless $handle && blessed $handle && $handle->isa('DBI::db') ;
-		$dbh = $handle;
-		$DSN = $setDSN;
-	};
-	if ($@) {
-		$logger->error($@);
-		return 0;
-	} else {
-		return 1;
-	}
-}
-
-# used to get the raw db handler, mainly for testability
-sub getConn {
-	return ($dbh, $DSN);
-}
-
 1;
 __END__
 
 =head1 NAME
 
+=encoding CP1252
+
 EAI::DB - Database wrapper functions (for DBI / DBD::ODBC)
 
 =head1 SYNOPSIS
 
- newDBH ($DB,$newDSN)
+ newDBH ($DB,$execute)
  beginWork ()
  commit ()
  rollback ()
@@ -553,8 +540,6 @@ EAI::DB - Database wrapper functions (for DBI / DBD::ODBC)
  storeInDB ($DB, $data)
  deleteFromDB ($DB, $data)
  updateInDB ($DB, $data)
- setConn ($handle, $DSN)
- getConn ()
 
 =head1 DESCRIPTION
 
@@ -564,81 +549,73 @@ EAI::DB contains all database related API-calls. This is for creating a database
 
 =over
 
-=item newDBH ($$)
+=item getConn
+
+returns the DBI handler and the DSN string to allow direct commands with the handler
+
+=item newDBH
 
 create a new handle for a database connection
 
  $DB .. hash with connection information like server, database
- $newDSN .. new DSN to be used for connection
+ $execute .. additional hash with execution information, mainly used for interpolation in DSN, especially environment: 'driver={SQL Server};Server=$DB->{server}{$execute->{env}};...'
 
-returns 0 on error, 1 if OK (handle is stored internally for further usage)
+ returns 0 on error, 1 if OK (handle is stored internally for further usage)
 
 =item beginWork
 
 start transaction in database
 
-returns 0 on error, 1 if OK
+ returns 0 on error, 1 if OK
 
 =item commit
 
 commit transaction in database
 
-returns 0 on error, 1 if OK
+ returns 0 on error, 1 if OK
 
 =item rollback
 
 roll back transaction in database
 
-returns 0 on error, 1 if OK
+ returns 0 on error, 1 if OK
 
-=item readFromDB ($$)
+=item readFromDB
 
 read data into array returned in $data
 
- $DB .. hash with information for the procedure, following keys:
+ $DB .. hash with information for the procedure
+ $data .. ref to array of hash values (as returned by fetchall_arrayref: $return[row_0based]->{"<fieldname>"}) for return values of query.
  $DB->{query} .. query string
  $DB->{columnnames} .. optionally return fieldnames of the query here
- $data .. ref to array of hash values (as returned by fetchall_arrayref: $return[row_0based]->{"<fieldname>"}) for return values of query.
+ returns 0 on error, 1 if OK
 
-returns 0 on error, 1 if OK
-
-=item readFromDBHash ($$)
+=item readFromDBHash
 
 read data into hash using column $DB->{keyfield} as the unique key for the hash (used for lookups), returned in $data
 
- $DB .. hash with information for the procedure, following keys:
+ $DB .. hash with information for the procedure
+ $data .. ref to hash of hash values (as returned by selectall_hashref: $return->{hashkey}->{"<fieldname>"}) for return values of query.
  $DB->{query} .. query string
  $DB->{columnnames} .. optionally return fieldnames of the query here
  $DB->{keyfield} .. field contained in the query string that should be used as the hashkey for the hash values of $data.
- $data .. ref to hash of hash values (as returned by selectall_hashref: $return->{hashkey}->{"<fieldname>"}) for return values of query.
+ returns 0 on error, 1 if OK
 
-returns 0 on error, 1 if OK
-
-=item doInDB ($;$)
+=item doInDB
 
 do general statement $DB->{doString} in database using optional parameters passed in array ref $DB->{parameters}, optionally passing back values in $data
 
- $DB .. hash with information for the procedure, following keys:
+ $DB .. hash with information for the procedure
+ $data .. optional: ref to array for return values of statement in $DB->{doString} (usually stored procedure).
  $DB->{doString} .. sql statement to be executed
  $DB->{parameters} .. optional: if there are placeholders defined in $DB->{doString} for parameters (?), then the values for these parameters are passed here.
- $data .. optional: ref to array for return values of statement in $DB->{doString} (usually stored procedure).
+ returns 0 on error, 1 if OK
 
-returns 0 on error, 1 if OK
-
-=item storeInDB ($$)
+=item storeInDB
 
 store row-based data into database, using insert or an "upsert" technique
 
- $DB .. hash with information for the procedure, following keys:
- $DB->{tableName} .. table where data should be inserted/updated (can have a prepended schema, separated with ".")
- $DB->{addID} .. add an additional, constant ID-field to the data (ref to hash: {"NameOfIDField" => "valueOfIDField"}), only one field/value pair is possible here
- $DB->{upsert} .. update a record after an insert failed due to an already existing primary key (-> "upsert")
- $DB->{primkey} .. WHERE clause (e.g. primID1 = ? AND primID2 = ?) for building the update statements
- $DB->{ignoreDuplicateErrs} .. if  $DB->{upsert} was not set and duplicate errors with inserts should be ignored
- $DB->{deleteBeforeInsertSelector} .. WHERE clause (e.g. col1 = ? AND col2 = ?) for deleting existing data before storing: all data that fullfills the criteria of this clause for values in the first data record of the data to be stored are being deleted (following the assumption that these criteria are the fulfilled for all records to be deleted)
- $DB->{incrementalStore} .. if set, then undefined (NOT empty ("" !) but undef) values are not being set to NULL but skipped for the insert/update statement
- $DB->{doUpdateBeforeInsert} .. if set, then the update in "upserts" is done BEFORE the insert, this is important for tables with an identity primary key and the inserting criterion is a/are different field(s).
- $DB->{debugKeyIndicator} .. key debug string (e.g. Key1 = ? Key2 = ?) to build debugging key information for error messages.
+ $DB .. hash with information for the procedure
  $data .. ref to array of hashes to be stored into database:
  $data = [
            {
@@ -652,27 +629,32 @@ store row-based data into database, using insert or an "upsert" technique
              ...
            },
          ];
+ $DB->{tableName} .. table where data should be inserted/updated (can have a prepended schema, separated with ".")
+ $DB->{addID} .. add an additional, constant ID-field to the data (ref to hash: {"NameOfIDField" => "valueOfIDField"}), only one field/value pair is possible here
+ $DB->{upsert} .. update a record after an insert failed due to an already existing primary key (-> "upsert")
+ $DB->{primkey} .. WHERE clause (e.g. primID1 = ? AND primID2 = ?) for building the update statements
+ $DB->{ignoreDuplicateErrs} .. if  $DB->{upsert} was not set and duplicate errors with inserts should be ignored
+ $DB->{deleteBeforeInsertSelector} .. WHERE clause (e.g. col1 = ? AND col2 = ?) for deleting existing data before storing: all data that fullfills the criteria of this clause for values in the first data record of the data to be stored are being deleted (following the assumption that these criteria are the fulfilled for all records to be deleted)
+ $DB->{incrementalStore} .. if set, then undefined (NOT empty ("" !) but undef) values are not being set to NULL but skipped for the insert/update statement
+ $DB->{doUpdateBeforeInsert} .. if set, then the update in "upserts" is done BEFORE the insert, this is important for tables with an identity primary key and the inserting criterion is a/are different field(s).
+ $DB->{debugKeyIndicator} .. key debug string (e.g. Key1 = ? Key2 = ?) to build debugging key information for error messages.
+ returns 0 on error, 1 if OK
 
-returns 0 on error, 1 if OK
-
-=item deleteFromDB ($$)
+=item deleteFromDB
 
 delete data identified by key-data in database
 
- $DB .. hash with information for the procedure, following keys:
+ $DB .. hash with information for the procedure
+ $data.. ref to hash of hash entries (as returned by selectall_hashref) having key values of records to be deleted
  $DB->{tableName} .. table where data should be deleted
  $DB->{keycol} .. a field name or a WHERE clause (e.g. primID1 = ? AND primID2 = ?) to find data that should be removed. A contained "?" specifies a WHERE clause that is simply used for a prepared statement.
- $data.. ref to hash of hash entries (as returned by selectall_hashref) having key values of records to be deleted
+ returns 0 on error, 1 if OK
 
-returns 0 on error, 1 if OK
-
-=item updateInDB ($$)
+=item updateInDB
 
 update data in database
 
- $DB .. hash with information for the procedure, following keys:
- $DB->{tableName} .. table where data should be updated
- $DB->{keycol} .. a field name or a WHERE clause (e.g. primID1 = ? AND primID2 = ?) to find data that should be updated. A contained "?" specifies a WHERE clause that is simply used for a prepared statement.
+ $DB .. hash with information for the procedure
  $data.. ref to hash of hash entries (as returned by selectall_hashref) having key values of records to be updated (keyval keys are artificial keys not being used for the update, they only uniquely identify the update records)
  $data = [ 'keyval1' => {
              'field1Name' => 'DS1field1Value',
@@ -684,19 +666,9 @@ update data in database
              ...
            },
          ];
-
-returns 0 on error, 1 if OK
-
-=item setConn ($$)
-
-set handle with externally created DBD::ODBC connection in case newDBH capabilities are not sufficient
-
- $handle .. ref to handle
- $setDSN .. DSN used in handle ((used for calls to newDBH)
- 
-=item getConn
-
-returns the DBI handler and the DSN string to allow direct commands with the handler
+ $DB->{tableName} .. table where data should be updated
+ $DB->{keycol} .. a field name or a WHERE clause (e.g. primID1 = ? AND primID2 = ?) to find data that should be updated. A contained "?" specifies a WHERE clause that is simply used for a prepared statement.
+ returns 0 on error, 1 if OK
 
 =back
 
