@@ -1,42 +1,39 @@
-package EAI::File 0.10;
-
-use strict; use feature 'unicode_strings'; use warnings; no warnings 'uninitialized';
-use Exporter qw(import);use Text::CSV();use Data::XLSX::Parser();use Spreadsheet::ParseExcel();use Spreadsheet::WriteExcel();use Excel::Writer::XLSX();use Data::Dumper qw(Dumper);use XML::LibXML();use XML::LibXML::Debugging();
-use Log::Log4perl qw(get_logger);use Time::localtime;use Scalar::Util qw(looks_like_number);use EAI::DateUtil;
+package EAI::File 0.2;
 
+use strict;
+use Text::CSV; use Data::XLSX::Parser; use Spreadsheet::ParseExcel; use Spreadsheet::WriteExcel; use Excel::Writer::XLSX; use Data::Dumper; use XML::LibXML; use XML::LibXML::Debugging;
+use Encode; use Cwd; use Log::Log4perl qw(get_logger); use Time::localtime; use Scalar::Util qw(looks_like_number); use EAI::DateUtil;
+use Exporter;
+our @ISA = qw(Exporter);
 our @EXPORT = qw(readText readExcel readXML writeText writeExcel);
 
-# get common read procedure parameters from $File config, used in readText, readExcel and readXML
-sub getcommon ($) {
-	my ($File) = @_;
+# get common read procedure parameters, used in readText, readExcel and readXML
+sub getcommon {
+	my ($File, $process, $lineProcessing, $fieldProcessing, $firstLineProc, $thousandsep, $decimalsep) = @_;
 	my $lineProcessing = $File->{lineCode};
 	my $fieldProcessing = $File->{fieldCode};
 	my $firstLineProc = $File->{firstLineProc};
 	my $thousandsep = "\\".$File->{format_thousandsep}; # add backslash to quote thousandsep for regexp (in case it is a ".")
 	my $decimalsep = "\\".$File->{format_decimalsep}; # add backslash to quote decimalsep for regexp (in case it is a ".")
-	my $skip = $File->{format_skip} if $File->{format_skip};
-	my $sep = $File->{format_sep} if $File->{format_sep};
-	$sep = $File->{format_defaultsep} if !$sep; # use default if not given
-	$sep = "\t" if !$sep; # use tab also no default
-	my @header = split $sep, $File->{format_header} if $File->{format_header};
-	my @targetheader = split $sep, $File->{format_targetheader} if $File->{format_targetheader};
-	get_logger()->debug("\$lineProcessing:$lineProcessing\n\$fieldProcessing:$fieldProcessing\n\$firstLineProc:$firstLineProc\n\$thousandsep:$thousandsep\n\$decimalsep:$decimalsep");
-	return ($lineProcessing,$fieldProcessing,$firstLineProc,$thousandsep,$decimalsep,$sep,$skip,\@header,\@targetheader);
+	get_logger()->debug("lineProcessing: $lineProcessing\nfieldProcessing: $fieldProcessing\nfirstLineProc:$firstLineProc\nthousandsep:$thousandsep\ndecimalsep:$decimalsep");
+	return ($lineProcessing, $fieldProcessing, $firstLineProc, $thousandsep, $decimalsep);
 }
 
 # read text files
-sub readText ($$$;$) {
-	my ($File,$data,$filenames,$redoSubDir) = @_;
+sub readText {
+	my ($File, $process, $filenames, $redoSubDir) = @_;
 	my $logger = get_logger();
+	my ($lineProcessing, $fieldProcessing, $firstLineProc, $thousandsep, $decimalsep) = getcommon($File, $process);
 	my @filenames = @{$filenames} if $filenames;
 	if (!@filenames) {
 		$logger->error("no filenames passed");
 		return 0;
 	}
 	# read format configuration
-	my ($lineProcessing,$fieldProcessing,$firstLineProc,$thousandsep,$decimalsep,$sep,$skip,$header,$targetheader) = getcommon($File);
-	my @header = @$header; my @targetheader = @$targetheader;
-	my ($poslen, $isFixLen); 
+	my ($poslen, $isFixLen, $skip, $sep); 
+	my (@header, @targetheader);
+	$skip = $File->{format_skip} if $File->{format_skip};
+	$sep = $File->{format_sep} if $File->{format_sep};
 	my $origsep = $sep;
 	if ($sep =~ /^fix/) {
 		# positions/length definitions from poslen definition: e.g. "format_poslen => [(0,3),(3,3)]"
@@ -49,14 +46,17 @@ sub readText ($$$;$) {
 			return 0;
 		}
 	}
+	@header = split $sep, $File->{format_header} if $File->{format_header};
+	@targetheader = split $sep, $File->{format_targetheader} if $File->{format_targetheader};
 	$Data::Dumper::Terse = 1;
 	$logger->debug("skip:$skip,sep:".Data::Dumper::qquote($origsep).",header:@header\ntargetheader:@targetheader");
 	$Data::Dumper::Terse = 0;
 	@targetheader = @header if !@targetheader; # if no specific targetheader defined use header instead
+
 	# read all files with same format
 	for my $filename (@filenames) {
 		$logger->debug("reading $redoSubDir$filename");
-		open (FILE, "<".$File->{format_encoding}, $redoSubDir.$filename) or do { #
+		open (FILE, "<".$File->{encoding}, $redoSubDir.$filename) or do { #
 			if (! -e $redoSubDir.$filename) {
 				$logger->error("no file $redoSubDir$filename to process...") unless ($File->{optional});
 				$logger->warn("no file $redoSubDir$filename found... ");
@@ -76,7 +76,7 @@ sub readText ($$$;$) {
 			my $newRecSep;
 			if ($File->{format_allowLinefeedInData}) {
 				# enable binmode and set line record separator to CRLF, so line feeds in values don't create artificial new lines/records
-				binmode(FILE, ":raw".$File->{format_encoding}); # raw so not to swallow CRLF
+				binmode(FILE, ":raw".$File->{encoding}); # raw so not to swallow CRLF
 				$newRecSep = "\015\012";
 				$logger->debug("binmode");
 			}
@@ -87,6 +87,7 @@ sub readText ($$$;$) {
 			$logger->debug("starting reading file $redoSubDir$filename ... ");
 			if ($firstLineProc) {
 				$_ = <FILE>;
+				$_ = encode('cp1252', $_) if $File->{encoding};
 				eval $firstLineProc;
 				$logger->error("eval firstLineProc: ".$firstLineProc.$@) if ($@);
 				$logger->debug("evaled: ".$firstLineProc);
@@ -99,15 +100,18 @@ sub readText ($$$;$) {
 					for (1 .. $skip) {$_ = <FILE>};
 				} else {
 					while (<FILE>) {
+						$_ = encode('cp1252', $_) if $File->{encoding};
 						last if /$skip/;
 					}
 				}
 			}
+
 			# iterate through all rows of file
 			my $lineno = 0;
 			my (@line,@previousline);
 LINE:
 			while (<FILE>) {
+				$_ = encode('cp1252', $_) if $File->{encoding};
 				chomp;
 				# in case lineProcessing or addtlProcessing needs access to whole row -> $rawline
 				my $rawline = $_;
@@ -132,17 +136,17 @@ LINE:
 				}
 				$lineno++;
 				next LINE if $line[0] eq "" and !$lineProcessing;
-				readRow($data,\@line,\@previousline,\@header,\@targetheader,$rawline,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno);
+				readRow($process,\@line,\@previousline,\@header,\@targetheader,$rawline,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno);
 			}
 		}
 		close FILE;
-		if (!$data and !$File->{emptyOK}) {
+		if (!$process->{data} and !$File->{emptyOK}) {
 			$logger->error("no data retrieved from file: $filename");
 			return 0;
 		}
 	}
-	$logger->trace("amount of rows:".scalar(@{$data})) if $logger->is_trace and $data;
-	$logger->trace(Dumper($data)) if $logger->is_trace;
+	$logger->trace("amount of rows:".scalar(@{$process->{data}})) if $logger->is_trace and $process->{data};
+	$logger->trace(Dumper($process->{data})) if $logger->is_trace;
 	return 1;
 }
 
@@ -199,6 +203,7 @@ sub row_handlerXLSX {
 		my $row = $cellDetail->{"row"};
 		my $col = $cellDetail->{"c"};
 		my $value = $cellDetail->{"v"};
+
 		if ($headerColumn{$col}) {
 			if (($stopOnEmptyValueColumn eq $col && !$value) || $stoppedOnEmptyValue) {
 				$logger->warn("empty cell in row $row / column $col and stopOnEmptyValueColumn is set to $col, skipping from here now") if !$stoppedOnEmptyValue; # pass warning only once
@@ -219,27 +224,26 @@ sub row_handlerXLSX {
 }
 
 # read Excel file (format depends on setting)
-sub readExcel ($$$;$) {
-	my ($File,$data,$filenames,$redoSubDir) = @_;
+sub readExcel {
+	my ($File, $process, $filenames, $redoSubDir) = @_;
 	my $logger = get_logger();
 	$stopOnEmptyValueColumn = $File->{format_stopOnEmptyValueColumn};
 	$stoppedOnEmptyValue = 0; # reset
+	my ($lineProcessing, $fieldProcessing, $firstLineProc, $thousandsep, $decimalsep) = getcommon($File, $process);
 	my @filenames = @{$filenames} if $filenames;
-	if (!@filenames) {
-		$logger->error("no filenames passed");
-		return 0;
-	}
+
 	# reset module global variables
 	undef %dateColumn;
 	undef %headerColumn;
+
 	# read format configuration
-	my ($lineProcessing,$fieldProcessing,$firstLineProc,$thousandsep,$decimalsep,$sep,$skip,$header,$targetheader) = getcommon($File);
-	my @header = @$header; my @targetheader = @$targetheader;
-	if (!@targetheader) {
-		$logger->error("no targetheader defined"); # targetheader has to be given, excel source header (@header) optional
-		return 0;
-	}
-	$logger->debug("skip: $skip,headerskip: ". $File->{format_headerskip}.", header: @header \ntargetheader: @targetheader\ndateColumns: ".($File->{format_dateColumns} ? @{$File->{format_dateColumns}} : "")."\nheaderColumns: ".($File->{format_headerColumns} ? @{$File->{format_headerColumns}} : ""));
+	my (@header, @targetheader);
+	my $sep = $File->{format_sep} if $File->{format_sep};
+	$sep = "\t" if !$sep;
+	$logger->error("no targetheader defined") if !$File->{format_targetheader}; # targetheader has to be given
+	@header = split $sep, $File->{format_header} if $File->{format_header}; # excel source header optional
+	@targetheader = split $sep, $File->{format_targetheader} if $File->{format_targetheader};
+	$logger->debug("skip: ". $File->{format_skip}."headerskip: ". $File->{format_headerskip}.", header: @header \ntargetheader: @targetheader\ndateColumns: ".($File->{format_dateColumns} ? @{$File->{format_dateColumns}} : "")."\nheaderColumns: ".($File->{format_headerColumns} ? @{$File->{format_headerColumns}} : ""));
 	# prepare dateColumn definition if needed/given
 	if ($File->{format_dateColumns} and ref($File->{format_dateColumns}) eq "ARRAY") {
 		for my $col (@{$File->{format_dateColumns}}) {
@@ -268,6 +272,7 @@ sub readExcel ($$$;$) {
 	}
 	$logger->debug("headerColumn:".Dumper(\%headerColumn).",dateColumn:".Dumper(\%dateColumn));
 	@header = @targetheader if !@header; # in the end only target header is important
+	
 	# read all files with same format
 	for my $filename (@filenames) {
 		my $startRow = 1; # starting data row
@@ -285,14 +290,16 @@ sub readExcel ($$$;$) {
 			$startRow = $startRowHeader + 1; # set to header following row if format_skip not defined and format_header given
 		}
 		# reset module global variables
-		%dataRows = ();
+		%dataRows = undef;
 		$maxRow = 1;
+
 		# check excel file existence
 		if (! -e $redoSubDir.$filename) {
 			$logger->error("no excel file ($filename) to process: $!") unless ($File->{optional});
 			$logger->warn("no file $redoSubDir$filename found"); 
 			return 0;
 		}
+
 		# read in excel file/sheet completely, both formats utilize read handlers (row_handlerXLSX or cell_handler)
 		my $parser;
 		if ($File->{format_xlformat} =~ /^xlsx$/i) {
@@ -300,6 +307,7 @@ sub readExcel ($$$;$) {
 			$parser = Data::XLSX::Parser->new;
 			$parser->open($redoSubDir.$filename);
 			$parser->add_row_event_handler(\&row_handlerXLSX);
+
 			if ($File->{format_worksheet}) {
 				$worksheet = $parser->workbook->sheet_id($File->{format_worksheet});
 				$logger->logdie("no worksheet found named ".$File->{format_worksheet}.", maybe try {format_worksheetID} (numerically ordered place)") if !$worksheet;
@@ -319,7 +327,7 @@ sub readExcel ($$$;$) {
 				NotSetCell  => 1
 			);
 			my $workbook = $parser->parse($redoSubDir.$filename);
-			if (!defined $workbook) {
+			if ( !defined $workbook ) {
 				$logger->error("excel parsing error: ".$parser->error());
 				return 0;
 			}
@@ -327,6 +335,7 @@ sub readExcel ($$$;$) {
 			$logger->error("unrecognised excel format passed in \$File->{format_xlformat}:".$File->{format_xlformat});
 			return 0;
 		}
+
 		# check header row if format_header given
 		if ($File->{format_header}) {
 			$logger->info("checking header info in row $startRowHeader");
@@ -362,46 +371,46 @@ LINE:
 					$line[$i] = $dataRows{$lineno}{$i+1};
 				}
 			}
-			readRow($data,\@line,\@previousline,\@header,\@targetheader,undef,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno);
+			readRow($process,\@line,\@previousline,\@header,\@targetheader,undef,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno);
 		}
 		close FILE;
-		if (scalar(@{$data}) == 0 and !$File->{emptyOK}) {
+		if (scalar(@{$process->{data}}) == 0 and !$File->{emptyOK}) {
 			$logger->error("Empty file: $filename, no data returned !!");
 			return 0;
 		}
 	}
-	$logger->trace("amount of rows: ".scalar(@{$data})) if $logger->is_trace;
-	$logger->trace(Dumper($data)) if $logger->is_trace;
+	$logger->trace("amount of rows: ".scalar(@{$process->{data}})) if $logger->is_trace;
+	$logger->trace(Dumper($process->{data})) if $logger->is_trace;
 	return 1;
 }
 
 # read XML file
-sub readXML ($$$;$) {
-	my ($File,$data,$filenames,$redoSubDir) = @_;
+sub readXML {
+	my ($File, $process, $filenames, $redoSubDir) = @_;
 	my $logger = get_logger();
+	my ($lineProcessing, $fieldProcessing, $firstLineProc, $thousandsep, $decimalsep) = getcommon($File, $process);
 	my @filenames = @{$filenames} if $filenames;
-	if (!@filenames) {
-		$logger->error("no filenames passed");
-		return 0;
-	}
+
 	# read format configuration
-	my ($lineProcessing,$fieldProcessing,$firstLineProc,$thousandsep,$decimalsep,$sep,$skip,$header,$targetheader) = getcommon($File);
-	my @header = @$header; my @targetheader = @$targetheader;
-	if (!@header) {
-		$logger->error("no header defined"); # targetheader has to be given, excel source header (@header) optional
-		return 0;
-	}
+	my (@header, @targetheader);
+	my $sep = $File->{format_sep};
+	$sep = "\t" if !$sep;
+	$logger->error("no header defined") if !$File->{format_header};
+	@header = split $sep, $File->{format_header};
+	@targetheader = split $sep, $File->{format_targetheader} if $File->{format_targetheader};
 	$Data::Dumper::Terse = 1;
 	$logger->debug("sep:".Data::Dumper::qquote($sep).",header:@header\ntargetheader:@targetheader");
 	$Data::Dumper::Terse = 0;
 	@targetheader = @header if !@targetheader; # if no specific targetheader defined use header instead
+
 	# read all files with same format
 	for my $filename (@filenames) {
 		if (! -e $redoSubDir.$filename) {
-			$logger->error("no XML file ($redoSubDir$filename) found to process") unless ($File->{optional});
+			$logger->error("no XML file ($filename) to process") unless ($File->{optional});
 			$logger->warn("file $redoSubDir$filename not found");
 			return 0;
 		}
+
 		my $xmldata = XML::LibXML->load_xml(location => $redoSubDir.$filename, no_blanks => 1);
 		my $xpc = XML::LibXML::XPathContext->new($xmldata);
 		if (ref($File->{format_namespaces}) eq 'HASH') {
@@ -438,9 +447,9 @@ sub readXML ($$$;$) {
 				}
 			}
 			$lineno++;
-			readRow($data,\@line,\@previousline,\@header,\@targetheader,undef,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno);
+			readRow($process,\@line,\@previousline,\@header,\@targetheader,undef,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno);
 		}
-		if (!$data and !$File->{emptyOK}) {
+		if (!$process->{data} and !$File->{emptyOK}) {
 			$logger->error("empty file: $filename, no data returned");
 			return 0;
 		}
@@ -449,8 +458,8 @@ sub readXML ($$$;$) {
 }
 
 # read row into final line hash (including special "hook" code)
-sub readRow ($$$$$$$$$$$) {
-	my ($data,$line,$previousline,$header,$targetheader,$rawline,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno) = @_;
+sub readRow {
+	my ($process,$line,$previousline,$header,$targetheader,$rawline,$lineProcessing,$fieldProcessing,$thousandsep,$decimalsep,$lineno) = @_;
 	my @line = @$line;
 	my @previousline = @$previousline;
 	my @header = @$header;
@@ -464,7 +473,6 @@ sub readRow ($$$$$$$$$$$) {
 	our $skipLineAssignment = 0; # can be set in fieldCode, to avoid further assignment to data.
 	# iterate through fields of current row
 	for (my $i = 0; $i < @line; $i++) {
-		#$logger->trace("field:$i, content:$line[$i]") if $logger->is_trace;
 		# first trim leading and trailing spaces
 		$line[$i] =~ s/^ *//;
 		$line[$i] =~ s/ *$//;
@@ -488,6 +496,7 @@ sub readRow ($$$$$$$$$$$) {
 		# field specific processing set, augments processing for a single specific field specified by targetheader...
 		if ($fieldProcessing->{$targetheader[$i]}) {
 			$logger->trace('BEFORE: $targetheader['.$i.']:'.$targetheader[$i].',$line{'.$targetheader[$i].']:'.$line{$targetheader[$i]}.',fieldProcessing{',$targetheader[$i],'}:'.$fieldProcessing->{$targetheader[$i]}) if $logger->is_trace;
+
 			eval $fieldProcessing->{$targetheader[$i]};
 			$logger->error('eval fieldProcessing->{',$targetheader[$i],'}: '.$fieldProcessing->{$targetheader[$i]}.$@) if ($@);
 			$logger->trace('AFTER: $targetheader['.$i.']:'.$targetheader[$i].',$line{'.$targetheader[$i].']:'.$line{$targetheader[$i]}.",\$skipLineAssignment: $skipLineAssignment, line: $lineno") if $logger->is_trace;
@@ -510,25 +519,28 @@ sub readRow ($$$$$$$$$$$) {
 		}
 	}
 	$logger->trace("line:\n".Dumper(\%line)) if $logger->is_trace and !$skipLineAssignment;
-	push @{$data}, \%line if %line and !$skipLineAssignment;
+	push @{$process->{data}}, \%line if %line and !$skipLineAssignment;
 }
 
 # write text file
-sub writeText ($$) {
-	my ($File,$data) = @_;
+sub writeText {
+	my ($File, $process) = @_;
 	my $logger = get_logger();
+	
 	my $filename = $File->{filename};
+	my $data = $process->{data};
 	if (ref($data) ne 'ARRAY') {
-		$logger->error("passed data in \$data is not a ref to array:".Dumper($data));
+		$logger->error("passed data in \$process is not a ref to array:".Dumper($process));
 		return 0;
 	}
 	# in case we need to print out csv/quoted values
 	my $sv = Text::CSV->new ({
-		binary    => 1,
-		auto_diag => 1,
-		sep_char  => $File->{format_sep},
-		eol => ($File->{format_eol} ? $File->{format_eol} : $/),
-	});
+			binary    => 1,
+			auto_diag => 1,
+			sep_char  => $File->{format_sep},
+			eol => ($File->{format_eol} ? $File->{format_eol} : $/),
+		});
+
 	my @columnnames; my @paddings;
 	if (ref($File->{columns}) eq 'ARRAY') {
 		@columnnames = @{$File->{columns}};
@@ -568,7 +580,7 @@ sub writeText ($$) {
 	}
 	# open file for writing
 	$logger->debug("writing to ".$filename);
-	open (FHOUT, ">".$File->{format_encoding},$filename) or do {
+	open (FHOUT, ">".$File->{encoding},$filename) or do {
 		$logger->error("file creation error: $!");
 		return 0;
 	};
@@ -584,6 +596,7 @@ sub writeText ($$) {
 			print FHOUT $headerRow."\n";
 		}
 	}
+	
 	# write data
 	$logger->trace("passed data:\n".Dumper($data)) if $logger->is_trace;
 	for (my $i=0; $i<scalar(@{$data}); $i++) {
@@ -595,7 +608,7 @@ sub writeText ($$) {
 		for my $colname (@columnnames) {
 			if (!$File->{columnskip}{$colname}) {
 				if (ref($row) ne "HASH") {
-					$logger->error("row passed in (\$data) is no ref to hash! should be \$VAR1 = {'key' => 'value', ...}:\n".Dumper($row));
+					$logger->error("row passed in (\$process->{data}) is no ref to hash! should be \$VAR1 = {'key' => 'value', ...}:\n".Dumper($row));
 					return 0;
 				}
 				my $value = $row->{$colname};
@@ -635,14 +648,16 @@ sub writeText ($$) {
 }
 
 # write Excel file
-sub writeExcel ($$) {
-	my ($File,$data) = @_;
+sub writeExcel {
+	my ($File, $process) = @_;
 	my $logger = get_logger();
 	
+	my $data = $process->{data};
 	if (ref($data) ne 'ARRAY') {
-		$logger->error("passed data in \$data is not a ref to array:".Dumper($data));
+		$logger->error("passed data in \$process is not a ref to array:".Dumper($process));
 		return 0;
 	}
+
 	my @columnnames;
 	if (ref($File->{columns}) eq 'HASH') {
 		@columnnames = map {$File->{columns}{$_}} sort keys %{$File->{columns}};
@@ -650,6 +665,7 @@ sub writeExcel ($$) {
 		$logger->error("no field information given (columns should be ref to hash)");
 		return 0;
 	}
+
 	my ($workbook,$worksheet);
 	if ($File->{format_xlformat} =~ /^xls$/i) {
 		$logger->debug("writing to xls format file ".$File->{filename});
@@ -669,6 +685,7 @@ sub writeExcel ($$) {
 	}
 	# Add a worksheet
 	$worksheet = $workbook->add_worksheet();
+
 	$logger->debug("fields: @columnnames");
 	my @headerRow;
 	for my $colname (@columnnames) {
@@ -682,6 +699,7 @@ sub writeExcel ($$) {
 			$worksheet->write(0,$col,$headerRow[$col]);
 		}
 	}
+	
 	# write data
 	$logger->trace("passed data:\n".Dumper($data)) if $logger->is_trace;
 	for (my $i=0; $i<scalar(@{$data}); $i++) {
@@ -691,7 +709,7 @@ sub writeExcel ($$) {
 		# chain all data in a row
 		for my $colname (@columnnames) {
 			if (!$File->{columnskip}{$colname}) {
-				$logger->error("row passed in (\$data) is no ref to hash! should be \$VAR1 = {'key' => 'value', ...}:\n".Dumper($row)) if (ref($row) ne "HASH");
+				$logger->error("row passed in (\$process->{data}) is no ref to hash! should be \$VAR1 = {'key' => 'value', ...}:\n".Dumper($row)) if (ref($row) ne "HASH");
 				my $value = $row->{$colname};
 				$logger->trace("\$value for \$colname $colname: $value") if $logger->is_trace;
 				if ($File->{addtlProcessingTrigger} && $File->{addtlProcessing}) {
@@ -718,11 +736,11 @@ EAI::File - read/parse Files from the filesystem or write to the filesystem
 
 =head1 SYNOPSIS
 
- readText ($File, $data, $filenames)
- readExcel ($File, $data, $filenames)
- readXML ($File, $data, $filenames)
- writeText ($File, $data)
- writeExcel ($File, $data)
+ readText ($File, $process, $filenames)
+ readExcel ($File, $process, $filenames)
+ readXML ($File, $process, $filenames)
+ writeText ($File, $process)
+ writeExcel ($File, $process)
 
 =head1 DESCRIPTION
 
@@ -732,53 +750,43 @@ EAI::File contains all file parsing API-calls. This is for reading plain text da
 
 =over
 
-=item readText ($$$)
+=item readText
 
 reads the defined text file with specified parameters into array of hashes (DB ready structure)
 
  $File      .. hash ref for File specific configuration
- $data      .. hash ref for returned data (hashkey "data" -> above mentioned array of hashes)
- $filenames .. array of file names, if explicit (given in case of mget and unpacked zip archives).
+ $process   .. hash ref for process specific configuration and returned data (hashkey "data" -> above mentioned array of hashes)
+ $filenames .. array of file names, if explizit (given in case of mget and unpacked zip archives).
 
-returns 0 on error, 1 if OK
-
-=item readExcel ($$$)
+=item readExcel
 
 reads the defined excel file with specified parameters into array of hashes (DB ready structure)
 
  $File      .. hash ref for File specific configuration
- $data      .. hash ref for returned data (hashkey "data" -> above mentioned array of hashes)
- $filenames .. array of file names, if explicit (given in case of mget and unpacked zip archives).
+ $process   .. hash ref for process specific configuration and returned data (hashkey "data" -> above mentioned array of hashes)
+ $filenames .. array of file names, if explizit (given in case of mget and unpacked zip archives).
 
-returns 0 on error, 1 if OK
-
-=item readXML ($$$)
+=item readXML
 
 reads the defined XML file with specified parameters into array of hashes (DB ready structure)
 
  $File      .. hash ref for File specific configuration
- $data      .. hash ref for returned data (hashkey "data" -> above mentioned array of hashes)
- $filenames .. array of filenamea, if explicit (given in case of mget and unpacked zip archives).
+ $process   .. hash ref for process specific configuration and returned data (hashkey "data" -> above mentioned array of hashes)
+ $filenames .. Array von filenamen, falls explizit array of file names, if explizit (given in case of mget and unpacked zip archives).
 
-returns 0 on error, 1 if OK
-
-=item writeText ($$)
+=item writeText
 
 writes a text file using specified parameters from array of hashes (DB structure) 
 
  $File      .. hash ref for File specific configuration
- $data      .. hash ref for returned data (hashkey "data" -> above mentioned array of hashes)
+ $process   .. hash ref for process specific configuration and returned data (hashkey "data" -> above mentioned array of hashes)
 
-returns 0 on error, 1 if OK
-
-=item writeExcel ($$)
+=item writeExcel
 
 writes an excel file using specified parameters from array of hashes (DB structure) 
 
  $File      .. hash ref for File specific configuration
- $data      .. hash ref for returned data (hashkey "data" -> above mentioned array of hashes)
-
-returns 0 on error, 1 if OK
+ $process   .. hash ref for process specific configuration and returned data (hashkey "data" -> above mentioned array of hashes)
 
 =back
 
